@@ -1,5 +1,6 @@
 from .backtrack_sampler import BacktrackSampler
 from .strategy.antislop_strategy import AntiSlopStrategy
 from .strategy.creative_writing_strategy import CreativeWritingStrategy
-__all__ = ['BacktrackSampler', 'AntiSlopStrategy',
+__all__ = ['BacktrackSampler',
+           'AntiSlopStrategy',
            'CreativeWritingStrategy']
