@@ -1,4 +1,4 @@
-from provider.base_provider import BaseProvider
+from .base_provider import BaseProvider
 import torch
 from typing import List
 from llama_cpp import Llama, BaseLlamaCache
@@ -18,7 +18,10 @@ class LlamacppProvider(BaseProvider):
         self.device = device
 
     def encode(self, text: str, add_special_tokens: bool = True) -> List[int]:
-        return self.llm.tokenize(text.encode("utf-8", errors="ignore"), add_bos=add_special_tokens, special=add_special_tokens)
+        return self.llm.tokenize(
+            text.encode("utf-8", errors="ignore"),
+            add_bos=add_special_tokens,
+            special=add_special_tokens)
 
     def decode(self, tokens: List[int]) -> str:
         return self.llm.detokenize(tokens).decode("utf-8", errors="ignore")
@@ -29,7 +32,6 @@ class LlamacppProvider(BaseProvider):
             prompt,
             max_tokens=1,
             echo=False,
-            logprobs=9999999999999999,
             temperature=1,
             top_p=1,
             top_k=9999999999999999,
@@ -45,7 +47,7 @@ class LlamacppProvider(BaseProvider):
     def crop_cache(self, idx: int) -> None:
         pass
 
-    def on_finish(self) -> None:
+    def reset(self) -> None:
         size = self.llm.cache.cache_size
         new_cache = type(self.llm.cache)(size)
         del self.llm.cache
