@@ -1,51 +1,61 @@
 import torch
-from strategy.base_strategy import BaseStrategy
+from .base_strategy import BaseStrategy
+from ..provider.base_provider import BaseProvider
 from typing import List
 
 
 class CreativeWritingStrategy(BaseStrategy):
-    def __init__(self, top_p_flat: float = 0.8, top_k_threshold_flat: int = 3, min_prob_second_highest: float = 0.25):
+    def __init__(
+        self, 
+        provider: BaseProvider, 
+        top_p_flat: float = 0.65, 
+        top_k_threshold_flat: int = 9, 
+        eos_penalty: float = 0.8
+    ):
         """
         top_p_flat: How many top tokens' probabilities sum up to this number?
-        top_k_threshold_flat: Minimum number of tokens required to sum up to top_p_flat in order
-            for the distribution to be considered flatten. The higher top_k_threshold_flat is, the less often
+        top_k_threshold_flat: If top_k_threshold_flat <= number of tokens that make up the top_p_flat value,
+            then the distribution is considered flattened. The higher top_k_threshold_flat is, the less often
             the algo will rollback.
-        min_prob_second_highest: What is the minimum probability the second most probable token token must have
-            in order to always be selected as next tokon.
+        eos_penalty: One commmon issue with this strategy is that it selects the eos too early when we ban
+            the most probable token. Therefore, we can apply a penalty to the eos token.
+            Values is between 0 and 1 where 1 means no penalty and 0 means eos is never selected 
+            (in which case the generation will stop via the max_length or max_new_tokens settings).
         """
+        self.eos_token = provider.get_eos_token_id()
         self.top_p_flat = top_p_flat
         self.top_k_threshold_flat = top_k_threshold_flat
-        self.min_prob_second_highest = min_prob_second_highest
-        self._is_flat = False
-        self._backtrack_data = None
-        self._keep_index = 0
+        self.eos_penalty = eos_penalty
+        self.reset()
 
     def get_keep_index(self) -> int:
         return self._keep_index
 
-    def on_logits(self, logits: torch.FloatTensor, continuation_tokens: List[int]) -> torch.FloatTensor:
-        # if we just backtracked, then make the natural highest probable token the chosen one
-        # else, make the chosen one the second natural highest probable token IF
-        # its probability is >= min_prob_second_highest
+    def on_logits(
+        self, logits: torch.FloatTensor, continuation_tokens: List[int]
+    ) -> torch.FloatTensor:
+        # if we just backtracked, then make sure the natural highest probable token will be selected
+        # else, make sure it won't
         if self._is_flat and self._backtrack_data != None:
             logits[:, self._backtrack_data[1]] = torch.finfo(logits.dtype).max
             self._backtrack_data = None
         else:
-            probabilities = torch.softmax(logits, dim=-1)
-            probabilities = probabilities.view(-1)
-            sorted_probs, sorted_indices = torch.sort(
-                probabilities, descending=True)
-            second_highest_prob = sorted_probs[1]
-            if second_highest_prob >= self.min_prob_second_highest:
-                logits[:, sorted_indices[1]] = torch.finfo(logits.dtype).max
-
+            max_index = torch.argmax(logits).item()
+            if(max_index != self.eos_token):
+                logits[:, max_index] = float("-inf")
+                
         return logits
 
-    def on_probs(self, probs: torch.FloatTensor, continuation_tokens: List[int]) -> torch.FloatTensor:
+    def on_probs(
+        self, probs: torch.FloatTensor, continuation_tokens: List[int]
+    ) -> torch.FloatTensor:
         self._is_flat = self._is_distribution_flat(probs)
+        probs[:, self.eos_token] = probs[:, self.eos_token] * self.eos_penalty
         return probs
 
-    def on_next_token(self, continuation_tokens: List[int], probs: torch.FloatTensor) -> None:
+    def on_next_token(
+        self, continuation_tokens: List[int], probs: torch.FloatTensor
+    ) -> None:
         latest_token = continuation_tokens[-1]
         highest_prob_token = torch.argmax(probs).item()
 
@@ -80,7 +90,13 @@ class CreativeWritingStrategy(BaseStrategy):
         cumulative_probs = torch.cumsum(sorted_probs, dim=0)
 
         # Find the number of top tokens needed to reach the cumulative probability threshold
-        num_top_tokens = torch.searchsorted(
-            cumulative_probs, self.top_p_flat).item() + 1
+        num_top_tokens = (
+            torch.searchsorted(cumulative_probs, self.top_p_flat).item() + 1
+        )
 
         return self.top_k_threshold_flat <= num_top_tokens
+    
+    def reset(self) -> None:
+        self._is_flat = False
+        self._backtrack_data = None
+        self._keep_index = 0
